rem Run Versioning consumption tests

\python31\python "C:\Users\Herm Fischer\Documents\mvsl\projects\Arelle\arelle\ArelleProject\arelle\MainCmdLine.py" --file "C:\Users\Herm Fischer\Documents\mvsl\projects\SEC\Local.Conformance\conformance\Private\Formula\Extension-Conformance\root\efm-15-101007\conf\testcases.xml" --gfm=us-gfm --validate --csvTestReport "C:\temp\US-GFM-test-report.csv" 1> "C:\temp\US-GFM-test-log.txt" 2>&1
